﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;
using ASPItalia.Apps4Windows.WinRT.Model;
using Windows.Data.Xml.Dom;
using Windows.UI.Notifications;
using Windows.UI.StartScreen;
using System.Diagnostics;

namespace ASPItalia.Apps4Windows.WinRT.Common
{
	/// <summary>
	/// ENG: Handles the tiles
	/// ITA: Gestione delle tile
	/// </summary>
	public static class TileHelper
	{
		/// <summary>
		/// ENG: Update the primary tile.
		/// ITA: Aggiorna la tile principale.
		/// </summary>
		/// <param name="content"></param>
		/// <param name="wideLogo"></param>
		/// <param name="logo"></param>
		public static void UpdatePrimaryTile(IPinnableContent content, Uri wideLogo, Uri logo)
		{
			// ENG: get the XML
			// ITA: recuperiamo l'XML
			XmlDocument xmlDoc = GetTileXml(content, wideLogo, logo);

			try
			{
				var tile = new TileNotification(xmlDoc);

				// ENG: we set the Tag property, so the tile can be uniquely identified and we can show the latest n tiles
				// ITA: impostiamo la proprietà Tag, così la tile sarà identificata univocamente sarà possibile ciclare tra le ultime n tile
				tile.Tag = content.UniqueID.GetHashCode().ToString();

				// ENG: let's update the tile
				// ITA: aggiorniamo la tile
				TileUpdateManager.CreateTileUpdaterForApplication().Update(tile);
			}
			catch (Exception ex)
			{
				Debug.WriteLine("Tile: " + ex.Message);
			}
		}

		/// <summary>
		/// ENG: Update the secondary tile
		/// ITA: Aggiorna la tile secondaria
		/// </summary>
		/// <param name="content"></param>
		/// <param name="wideLogo"></param>
		/// <param name="logo"></param>
		public static void UpdateSecondaryTile(IPinnableContent content, Uri wideLogo, Uri logo)
		{
			string pinId = PinHelper.GetPinId(content);

			// ENG: let's check if the tile exists
			// ITA: secondary tile, controlliamo prima che esista
			if (SecondaryTile.Exists(pinId))
			{
				// ENG: get the XML
				// ITA: recuperiamo l'XML
				XmlDocument xmlDoc = GetTileXml(content, wideLogo, logo);

				// ENG: let's update the tile
				// ITA: aggiorniamo la tile
				var secTile = TileUpdateManager.CreateTileUpdaterForSecondaryTile(pinId);
				secTile.Update(new TileNotification(xmlDoc));
			}
		}

		private static XmlDocument GetTileXml(IPinnableContent content, Uri wideLogo, Uri logo)
		{
			// ENG: the tile is created using LINQ TO XML
			// ITA: componiamo la tile con LINQ to XML
			var tileXml = new XDocument(new XElement("tile",
							new XElement("visual",
								new XElement("binding",
									new XAttribute("template", "TileWideImageAndText01"),
                                        new XElement("image", new XAttribute("id", "1"), new XAttribute("src", content.Image ?? wideLogo)),
										new XElement("text", new XAttribute("id", "1"), content.Title),
										new XElement("text", new XAttribute("id", "2"), content.Subtitle)
								),
								new XElement("binding",
									new XAttribute("template", "TileSquarePeekImageAndText04"),
                                        new XElement("image", new XAttribute("id", "1"), new XAttribute("src", content.Image ?? logo)),
										new XElement("text", new XAttribute("id", "1"), content.Title),
										new XElement("text", new XAttribute("id", "2"), content.Subtitle)
										)
								)
							));

			var xmlDoc = new XmlDocument();
			xmlDoc.LoadXml("<?xml version=\"1.0\" encoding=\"utf-8\" ?>" + tileXml.ToString());
			return xmlDoc;
		}
	}
}
