﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using ASPItalia.Apps4Windows.WinRT.Common;
using ASPItalia.Apps4Windows.WinRT.Extensions;
using ASPItalia.Apps4Windows.WinRT.Model;
using ASPItalia.Apps4Windows.WinRT.Services;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

namespace ASPItalia.Apps4Windows.WinRT.Pages
{
    public sealed partial class GridPage : ASPItalia.Apps4Windows.WinRT.Common.LayoutAwarePage
    {
        public GridPage()
        {
            this.InitializeComponent();
        }

        protected async override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);

			// ENG: the paramaters are deserialized from a string
            // ITA: Deserializzo i parametri passati come stringa
            object[] parameters = Utils.DeserializeValues((string)e.Parameter);
            
			// ENG: get section and its ID
			// ITA: Recupero nome sezione e l'id della risorsa
            this.sectionName = (SectionNames)parameters[0];

            await RefreshSection();
        }

        public override void Refresh()
        {
            base.Refresh();

#pragma warning disable 4014
            RefreshSection();
#pragma warning restore 4014
        }

        private async System.Threading.Tasks.Task RefreshSection()
        {
            try
            {
                try
                {
					// ENG: show the progress ring
                    // ITA: Mostro il progress di caricamento
                    this.ShowProgress = true;

					// ENG: loads section and its items
                    // ITA: Carico la sezione con i relaivi elementi
                    this.Section = await SectionsHelper.GetAndPrepareSessionAsync(sectionName);
                }
                finally
                {
					// ENG: the progress is collapsed anyway
                    // ITA: Qualunque cosa succeda nascondo il progress di caricamento
                    this.ShowProgress = false;
                }
                var isPinned = await PinHelper.IsPinned(Section);
                UpdateAppBar(isPinned);
            }
            catch (Exception ex)
            {
				// ENG: load the info from the local storage and set the title
                // ITA: carichiamo le info sulla sezione da locale, per passare il titolo
                ILoader loader;
                Section section;
                Repository.GetSectionAndLoader(sectionName, out section, out loader);
                Section = section;

                HandleError(ex);
            }
        }

        Section _section;
        private SectionNames sectionName;
        public Section Section
        {
            get
            {
                return _section;
            }
            set
            {
                base.SetProperty(ref _section, value);
            }
        }

        void ItemView_ItemClick(object sender, ItemClickEventArgs e)
        {
            var item = ((SectionItem)e.ClickedItem);
            this.Frame.NavigateToDetails(item);
        }

        private async Task RemovePin()
        {
            await PinHelper.RemovePin(Section);
        }

        private async Task Pin()
        {
            try
            {
                this.BottomAppBar.IsSticky = true;
                await PinHelper.Pin(Section, this.PinButton);
            }
            finally
            {
                this.BottomAppBar.IsSticky = false;
            }
        }

        private void UpdateAppBar(bool isPinned)
        {
            PinButton.Visibility = !isPinned ? Visibility.Visible : Visibility.Collapsed;
            UnPinButton.Visibility = isPinned ? Visibility.Visible : Visibility.Collapsed;
        }

        private async void PinOrUnPin(object sender, RoutedEventArgs e)
        {
            var isPinned = await PinHelper.IsPinned(Section);
            if (isPinned)
                await RemovePin();
            else
                await Pin();

            UpdateAppBar(!isPinned);
        }

        private void Refresh_Click(object sender, RoutedEventArgs e)
        {
            this.Refresh();
        }
    }
}
