﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;

namespace ASPItalia.Apps4Windows.WinRT.Common
{
	/// <summary>
	/// ENG: This gris works with VariableSizedWrapGrid to set VariableSizedWrapGrid.ColumnSpan e VariableSizedWrapGrid.RowSpan
	/// The IFreeResizable must be implemented by the items used in binding and set the panel alignment.
	/// ITA: Griglia che lavora con VariableSizedWrapGrid per impostare VariableSizedWrapGrid.ColumnSpan e VariableSizedWrapGrid.RowSpan
	/// L'interfaccia IFreeResizable che ogni elemento in binding può implementare, dice come allinearsi sul pannello
	/// </summary>
	public class VariableGridView : GridView
	{
		protected override void PrepareContainerForItemOverride(DependencyObject element, object item)
		{
			IFreeResizable free = item as IFreeResizable;
			if (free != null)
			{
				if (free.ColumnSpan.HasValue)
					element.SetValue(VariableSizedWrapGrid.ColumnSpanProperty, free.ColumnSpan.Value);

				if (free.RowSpan.HasValue)
					element.SetValue(VariableSizedWrapGrid.RowSpanProperty, free.RowSpan.Value);

				if (free.FillVertical)
					element.SetValue(GridViewItem.VerticalContentAlignmentProperty, VerticalAlignment.Stretch);

				if (free.FillHorizontal)
					element.SetValue(GridViewItem.HorizontalContentAlignmentProperty, HorizontalAlignment.Stretch);
			}

			base.PrepareContainerForItemOverride(element, item);
		}

	}

	/// <summary>
	/// ENG: This interface represents the items loaded by the VariableGridView
	/// ITA: Interfaccia per rappresentare gli elementi caricati sulla VariableGridView
	/// </summary>
	public interface IFreeResizable
	{
		/// <summary>
		/// ENG: Set the rows (optional)
		/// ITA: Restituisce quante righe occupare (facoltativo)
		/// </summary>
		int? RowSpan { get; }
		/// <summary>
		/// ENG: Set the columns (optional)
		/// ITA: Restituisce quante colonne occupare (facoltativo)
		/// </summary>
		int? ColumnSpan { get; }
		/// <summary>
		/// ENG: indicates if the element should fill its horizontal space
		/// ITA: Indica se l'elemento deve occupare tutto lo spazio che gli è stato dato orizzontalmente
		/// </summary>
		bool FillHorizontal { get; }
		/// <summary>
		/// /// ENG: indicates if the element should fill its vertical space
		/// ITA: Indica se l'elemento deve occupare tutto lo spazio che gli è stato dato verticalmente
		/// </summary>
		bool FillVertical { get; }
	}
}
