﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ASPItalia.Apps4Windows.WinRT.Services;

namespace ASPItalia.Apps4Windows.WinRT.Model
{
	/// <summary>
	/// ENG: this class contains the section.
	/// ITA: Indica una sezione di contenuti all'interno dell'applicazione.
	/// </summary>
	public class Section : IPinnableContent
	{
		public Section(SectionNames name, string title, SectionTypeEnum type, LayoutTypeEnum layout = LayoutTypeEnum.Classic)
		{
			Items = new ObservableCollection<SectionItem>();
			Title = title;
			Type = type;
			Name = name;
			Layout = layout;
		}

		public string Title { get; set; }
		public string Subtitle { get; set; }
		public string Description { get; set; }

		public SectionNames Name { get; private set; }

		public ObservableCollection<SectionItem> Items { get; protected set; }

		public SectionTypeEnum Type { get; private set; }

		public LayoutTypeEnum Layout { get; private set; }

		public Uri Image
		{
			get
			{
				// prendiamo la prima prima immagine degli items o quella di default
				return this.Items.Where(x => x.Image != null).Select(x => x.Image).FirstOrDefault();
			}
		}

		public string UniqueID
		{
			get { return this.Name.ToString(); }
		}

		public bool HasError { get; set; }
	}

	/// <summary>
	/// ENG: Section types. Very section has its own template.
	/// ITA: Le tipologie di sezioni supportate. Ogni sezione ha un suo template grafico.
	/// </summary>
	public enum SectionTypeEnum
	{
		PhotoGallery,
		List,
		Grid,
		News,
        VideoGallery,
	}

	/// <summary>
	/// ENG: Home page layouts
	/// ITA: I tipi di layout supportati in homepage.
	/// </summary>
	public enum LayoutTypeEnum
	{
		Classic,
		Grid,
		Horizontal,
		Alternative
	}
}
