using System;
using System.Linq;
using System.Collections.Generic;
using ASPItalia.Apps4Windows.WinRT.Common;
using ASPItalia.Apps4Windows.WinRT.Extensions;
using ASPItalia.Apps4Windows.WinRT.Model;
using ASPItalia.Apps4Windows.WinRT.Services;
using Windows.UI.ViewManagement;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;
using Windows.UI.Core;
using Windows.ApplicationModel.DataTransfer;
using Windows.Foundation;
using Windows.Storage.Streams;

namespace ASPItalia.Apps4Windows.WinRT.Pages
{
    public sealed partial class ListPage : ASPItalia.Apps4Windows.WinRT.Common.LayoutAwarePage
    {
        public ListPage()
        {
            this.InitializeComponent();
        }

        private bool _canGoBack;
        public bool CanGoBack
        {
            get { return _canGoBack; }
            set { this.SetProperty(ref _canGoBack, value); }
        }

        private Section _section;
        private SectionNames sectionName;
        private string itemId;
        public Section Section
        {
            get { return _section; }
            set { this.SetProperty(ref _section, value); }
        }

        #region Page state management

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);

            SetupShareContract();
        }

        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            base.OnNavigatedFrom(e);

            UnSetupShareContract();
        }

        /// <summary>
        /// Populates the page with content passed during navigation.  Any saved state is also
        /// provided when recreating a page from a prior session.
        /// </summary>
        /// <param name="navigationParameter">The parameter value passed to
        /// <see cref="Frame.Navigate(Type, Object)"/> when this page was initially requested.
        /// </param>
        /// <param name="pageState">A dictionary of state preserved by this page during an earlier
        /// session.  This will be null the first time a page is visited.</param>
        protected async override void LoadState(Object navigationParameter, Dictionary<String, Object> pageState)
        {
			// ENG: the paramaters are deserialized from a string
			// ITA: Deserializzo i parametri passati come stringa
            object[] parameters = Utils.DeserializeValues((string)navigationParameter);

			// ENG: get section and its ID
			// ITA: Recupero nome sezione e l'id della risorsa
            this.sectionName = (SectionNames)parameters[0];
            if (parameters.Length > 1)
            {
                this.itemId = parameters[1] as string;
            }

            await RefreshSection();

			// ENG: get the element from the state
			// ITA: Recupero dallo stato l'elemento su cui era posizionato
            if (pageState == null)
            {
                // When this is a new page, select the first item automatically unless logical page
                // navigation is being used (see the logical page navigation #region below.)
                if (this.itemsViewSource.View != null)
                {
                    if (!this.UsingLogicalPageNavigation())
                    {
                        this.itemsViewSource.View.MoveCurrentToFirst();
                    }
                    else
                    {
                        this.itemsViewSource.View.MoveCurrentTo(null);
                    }
                }
                
                
            }
            else
            {
                // Restore the previously saved state associated with this page
                if (pageState.ContainsKey("SelectedItem") && this.itemsViewSource.View != null)
                {
                    this.itemId = pageState["SelectedItem"] as string;
                }
            }

            if (this.Section != null && this.Section.Items != null && !string.IsNullOrEmpty(this.itemId))
            {
                var selectedItem = this.Section.Items
                    .FirstOrDefault(x => x.InnerItem != null && x.InnerItem.Id == itemId);

                if (selectedItem != null)
                    this.itemsViewSource.View.MoveCurrentTo(selectedItem);
            }
        }

        public override void Refresh()
        {
            base.Refresh();

#pragma warning disable 4014
            RefreshSection();
#pragma warning restore 4014
        }

        private async System.Threading.Tasks.Task RefreshSection()
        {
            try
            {
                try
                {
					// ENG: show the progress ring
					// ITA: Mostro il progress di caricamento
                    this.ShowProgress = true;

					// ENG: loads section and its items
					// ITA: Carico la sezione con i relaivi elementi
                    this.Section = await SectionsHelper.GetAndPrepareSessionAsync(sectionName);
                }
                finally
                {
					// ENG: the progress is collapsed anyway
					// ITA: Qualunque cosa succeda nascondo il progress di caricamento
                    this.ShowProgress = false;
                }
            }
            catch (Exception e)
            {
                HandleError(e);
            }
        }

        private void SetupShareContract()
        {
			// ENG: share contract setup
			// ITA: impostiamo il contratto di share
            DataTransferManager.GetForCurrentView().DataRequested += new TypedEventHandler<DataTransferManager, DataRequestedEventArgs>(this.DataRequested);
        }

        private void UnSetupShareContract()
        {
			// ENG: to avoid leaks, the share contract is de-registered
			// ITA: de-registriamo il contratto di share, per evitare leak
            DataTransferManager.GetForCurrentView().DataRequested -= new TypedEventHandler<DataTransferManager, DataRequestedEventArgs>(this.DataRequested);
        }

        private void DataRequested(DataTransferManager sender, DataRequestedEventArgs e)
        {
            SectionItem item = this.itemsViewSource.View.CurrentItem as SectionItem;
            Utils.ShareDetail(e, item);
        }

        /// <summary>
        /// Preserves state associated with this page in case the application is suspended or the
        /// page is discarded from the navigation cache.  Values must conform to the serialization
        /// requirements of <see cref="SuspensionManager.SessionState"/>.
        /// </summary>
        /// <param name="pageState">An empty dictionary to be populated with serializable state.</param>
        protected override void SaveState(Dictionary<String, Object> pageState)
        {
            if (this.itemsViewSource.View != null)
            {
                SectionItem selectedItem = this.itemsViewSource.View.CurrentItem as SectionItem;

                if (selectedItem != null && !string.IsNullOrWhiteSpace(selectedItem.InnerItem.Id))
                {
                    var id = selectedItem.InnerItem.Id;

                    pageState["SelectedItem"] = id;
                }
            }
        }

        #endregion

        #region Logical page navigation

        // Visual state management typically reflects the four application view states directly
        // (full screen landscape and portrait plus snapped and filled views.)  The split page is
        // designed so that the snapped and portrait view states each have two distinct sub-states:
        // either the item list or the details are displayed, but not both at the same time.
        //
        // This is all implemented with a single physical page that can represent two logical
        // pages.  The code below achieves this goal without making the user aware of the
        // distinction.

        /// <summary>
        /// Invoked to determine whether the page should act as one logical page or two.
        /// </summary>
        /// <param name="viewState">The view state for which the question is being posed, or null
        /// for the current view state.  This parameter is optional with null as the default
        /// value.</param>
        /// <returns>True when the view state in question is portrait or snapped, false
        /// otherwise.</returns>
        private bool UsingLogicalPageNavigation(ApplicationViewState? viewState = null)
        {
            if (viewState == null) viewState = ApplicationView.Value;
            return viewState == ApplicationViewState.FullScreenPortrait ||
                viewState == ApplicationViewState.Snapped;
        }

        /// <summary>
        /// Invoked when an item within the list is selected.
        /// </summary>
        /// <param name="sender">The GridView (or ListView when the application is Snapped)
        /// displaying the selected item.</param>
        /// <param name="e">Event data that describes how the selection was changed.</param>
        void ItemListView_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            // Invalidate the view state when logical page navigation is in effect, as a change
            // in selection may cause a corresponding change in the current logical page.  When
            // an item is selected this has the effect of changing from displaying the item list
            // to showing the selected item's details.  When the selection is cleared this has the
            // opposite effect.
            if (this.UsingLogicalPageNavigation())
#pragma warning disable 4014
                this.Dispatcher.RunAsync(CoreDispatcherPriority.Normal, 
                    () => this.InvalidateVisualState());
#pragma warning restore 4014
        }

        /// <summary>
        /// Invoked when the page's back button is pressed.
        /// </summary>
        /// <param name="sender">The back button instance.</param>
        /// <param name="e">Event data that describes how the back button was clicked.</param>
        protected override void GoBack(object sender, RoutedEventArgs e)
        {
            if (this.UsingLogicalPageNavigation() && itemListView.SelectedItem != null)
            {
                // When logical page navigation is in effect and there's a selected item that
                // item's details are currently displayed.  Clearing the selection will return to
                // the item list.  From the user's point of view this is a logical backward
                // navigation.
                this.itemListView.SelectedItem = null;
            }
            else
            {
                // When logical page navigation is not in effect, or when there is no selected
                // item, use the default back button behavior.
                base.GoBack(sender, e);
            }
        }

        /// <summary>
        /// Invoked to determine the name of the visual state that corresponds to an application
        /// view state.
        /// </summary>
        /// <param name="viewState">The view state for which the question is being posed.</param>
        /// <returns>The name of the desired visual state.  This is the same as the name of the
        /// view state except when there is a selected item in portrait and snapped views where
        /// this additional logical page is represented by adding a suffix of _Detail.</returns>
        protected override string DetermineVisualState(ApplicationViewState viewState)
        {
            // Update the back button's enabled state when the view state changes
            var logicalPageBack = this.UsingLogicalPageNavigation(viewState) && this.itemListView.SelectedItem != null;
            var physicalPageBack = this.Frame != null && this.Frame.CanGoBack;
            this.CanGoBack = logicalPageBack || physicalPageBack;

            // Determine visual states for landscape layouts based not on the view state, but
            // on the width of the window.  This page has one layout that is appropriate for
            // 1366 virtual pixels or wider, and another for narrower displays or when a snapped
            // application reduces the horizontal space available to less than 1366.
            if (viewState == ApplicationViewState.Filled ||
                viewState == ApplicationViewState.FullScreenLandscape)
            {
                var windowWidth = Window.Current.Bounds.Width;
                if (windowWidth >= 1366) return "FullScreenLandscapeOrWide";
                return "FilledOrNarrow";
            }

            // When in portrait or snapped start with the default visual state name, then add a
            // suffix when viewing details instead of the list
            var defaultStateName = base.DetermineVisualState(viewState);
            return logicalPageBack ? defaultStateName + "_Detail" : defaultStateName;
        }

        #endregion
    }
}
